/*
 * Copyright (c) 2017, MegaEase
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package websocketserver

import (
	"github.com/megaease/easegress/pkg/supervisor"
)

const (
	// Category is the category of WebSocketServer.
	Category = supervisor.CategoryBusinessController

	// Kind is the kind of WebSocketServer.
	Kind = "WebSocketServer"
)

func init() {
	supervisor.Register(&WebSocketServer{})
}

type (
	// WebSocketServer implements WebSocket protocol in EG.
	WebSocketServer struct {
		superSpec *supervisor.Spec
		spec      *Spec
		proxy     *Proxy
	}
)

// Category returns the category of WebsocketServer.
func (ws *WebSocketServer) Category() supervisor.ObjectCategory {
	return Category
}

// Kind returns the kind of WebsocketServer.
func (ws *WebSocketServer) Kind() string {
	return Kind
}

// DefaultSpec returns the default spec of WebSocketServer.
func (ws *WebSocketServer) DefaultSpec() interface{} {
	return &Spec{}
}

// Validate validates the websocketserver structure.
func (ws *WebSocketServer) Validate() error {
	return nil
}

// Init initializes Function.
func (ws *WebSocketServer) Init(superSpec *supervisor.Spec) {
	ws.superSpec, ws.spec = superSpec, superSpec.ObjectSpec().(*Spec)
	ws.reload()
}

// Inherit inherits previous generation of WebSocketServer.
func (ws *WebSocketServer) Inherit(superSpec *supervisor.Spec, previousGeneration supervisor.Object) {
	previousGeneration.Close()
	ws.Init(superSpec)
}

func (ws *WebSocketServer) reload() {
	ws.proxy = newProxy(ws.superSpec)
}

// Status returns Status generated by proxy.
func (ws *WebSocketServer) Status() *supervisor.Status {
	return &supervisor.Status{}
}

// Close closes WebSocketServer.
func (ws *WebSocketServer) Close() {
	ws.proxy.Close()
}
